/*
 * Copyright 2012 Fotonic
 *
 * License: Public Domain
 *
 */

//
// example program that shows how to use the FZ-API to control the camera.
// the code
//  - connect to the first camera found
//  - get some version info and print it
//  - set some parameters
//  - start the camera
//  - get a number of frames
//  - save one image to disc in 3DD format
//  - stop the camera
//  - disconnect and exit
//

#include <stdlib.h>
#include <stdio.h>

#include "fz_api.h"

#define MAX_DEVICES 40

// write image data to disc, replayable by 3DDisplay
bool SaveImageAs3DD(short *pPixels, FZ_FRAME_HEADER *pstFrmHdr)
{
	// writes one image to one .3dd file (binary header+raw intel fmt)
	bool bResult = false;
	int n;
	int iWidth = pstFrmHdr->ncols;
	int iHeight = pstFrmHdr->nrows;
	int iPixelSize = pstFrmHdr->bytesperpixel;

	char szFilename[256];
	sprintf(szFilename, "fz_example.3dd");

	FILE *pFile = fopen(szFilename, "wb");
	if(!pFile) return false;

	// write header as it is
	n = (int)fwrite(pstFrmHdr, sizeof(FZ_FRAME_HEADER), 1, pFile);
	if(n!=1) goto error;

	// write image data as it is
	n = (int)fwrite(pPixels, iWidth*iHeight*iPixelSize, 1, pFile);
	if(n!=1) goto error;

	bResult = true;
error:
	fclose(pFile);
	return bResult;
}


FZ_FRAME_HEADER stFrameHdr;
short aImage[640*480*4];

void FZLogCallback(char *szMetadata, char *szMessage)
{
	printf("LOG CALLBACK: [%s] %s\n", szMetadata, szMessage);
}

int main()
{
	FZ_DEVICE_INFO aDeviceInfo[MAX_DEVICES];
	FZ_Result iResult;
	FZ_Device_Handle_t hDevice;
	FZ_CmdRespCode_t iRespCode;
	int iRespBytes;
	char szInfo[128];
	uint16_t iDeviceType;
	uint16_t iMode;
	uint16_t iShutterMs10 = 20; //2.0ms
	uint16_t iFPS = 40;
	bool bSaveOK;

	// init FZ_API
	FZ_Init();

	// set desired FZ-API logging
	int iFlags = FZ_LOG_TO_STDOUT;
	iFlags |= FZ_LOG_ERROR | FZ_LOG_WARN | FZ_LOG_INFO;
	FZ_SetLogging(iFlags, NULL, FZLogCallback);

	// find connected sensors (enumerate)
	int iNumDevices = MAX_DEVICES;
	iResult = FZ_EnumDevices2(aDeviceInfo, &iNumDevices);
	if(iResult==FZ_TOO_MANY_DEVICES) iResult = FZ_Success;
	if(iResult!=FZ_Success || iNumDevices<1) {
		if(iResult!=FZ_Success) printf("ERROR: FZ_EnumDevices2 code 0x%02x\n", iResult);
		else printf("ERROR: No FZ devices are found\n");
		goto error;
	}
	printf("Found %d FZ devices\n", iNumDevices);

	// at least one device is found, open
	iResult = FZ_Open(aDeviceInfo[0].szPath, 0, &hDevice);
	if(iResult!=FZ_Success) {
		printf("ERROR: FZ_Open (code 0x%02x)\n", iResult);
		goto error;
	}

	// get device type
	iRespBytes = sizeof(iDeviceType);
	iResult = FZ_IOCtl(hDevice,
		CMD_DE_GET_DEVICE_TYPE, NULL, 0,
		&iRespCode, &iDeviceType, &iRespBytes);
	if(iRespCode != (int)R_CMD_DE_ACK) iDeviceType = FZ_DEVICE_TYPE_JAGUAR;
	printf("DeviceType: %d\n", iDeviceType);

	// get and print info strings
	iRespBytes = 127;
	iResult = FZ_IOCtl(hDevice,
		CMD_API_GET_VERSION, NULL, 0,
		&iRespCode, szInfo, &iRespBytes);
	if(iResult!=FZ_Success) goto error;
	printf("FZ_API version: %s\n", szInfo);
	iRespBytes = 127;
	iResult = FZ_IOCtl(hDevice,
		CMD_DE_GET_VERSION, NULL, 0,
		&iRespCode, szInfo, &iRespBytes);
	if(iResult!=FZ_Success) goto error;
	printf("DE version: %s\n", szInfo);
	iRespBytes = 127;
	iResult = FZ_IOCtl(hDevice,
		CMD_CA_GET_VERSION, NULL, 0,
		&iRespCode, szInfo, &iRespBytes);
	if(iResult!=FZ_Success) goto error;
	printf("CA version: %s\n", szInfo);
	iRespBytes = 127;
	iResult = FZ_IOCtl(hDevice,
		CMD_DE_GET_PCODE, NULL, 0,
		&iRespCode, szInfo, &iRespBytes);
	if(iResult!=FZ_Success) goto error;
	printf("Product code: %s\n", szInfo);
	iRespBytes = 127;
	iResult = FZ_IOCtl(hDevice,
		CMD_DE_GET_UNIT_NO, NULL, 0,
		&iRespCode, szInfo, &iRespBytes);
	if(iResult!=FZ_Success) goto error;
	printf("S/N: %s\n", szInfo);

	// set mode
	if(iDeviceType==FZ_DEVICE_TYPE_JAGUAR)          iMode = DE_MODE_BM_TEMPORAL;
	else if(iDeviceType==FZ_DEVICE_TYPE_PANASONIC)  iMode = DE_MODE_PA_Z; 
	else if(iDeviceType==FZ_DEVICE_TYPE_PRIMESENSE) iMode = DE_MODE_640X480_30;
	else if(iDeviceType==FZ_DEVICE_TYPE_PRIMESENSE_C)  iMode = DE_MODE_640X480_640X480; 
	else if(iDeviceType==FZ_DEVICE_TYPE_PRIMESENSE_N) iMode = DE_MODE_640X480_640X480;
	else {
		printf("ERROR: Unknown DeviceType (%d)\n", iDeviceType);
		goto error;
	}
	iResult = FZ_IOCtl(hDevice,
		CMD_DE_SET_MODE, &iMode, sizeof(iMode),
		&iRespCode, NULL, NULL);
	if( iResult!=FZ_Success ) {
		printf("ERROR: FZ_IOCtl CMD_DE_SET_MODE failed (code 0x%02x)\n", iResult);
		goto error;
	}
	// set shutter
	iResult = FZ_IOCtl(hDevice,
		CMD_DE_SET_SHUTTER, &iShutterMs10, sizeof(iShutterMs10),
		&iRespCode, NULL, NULL);
	if( iResult!=FZ_Success ) {
		printf("ERROR: FZ_IOCtl CMD_DE_SET_SHUTTER failed (code 0x%02x)\n", iResult);
		goto error;
	}
	// set frame rate
	iResult = FZ_IOCtl(hDevice,
		CMD_DE_SET_FPS, &iFPS, sizeof(iFPS),
		NULL, NULL, NULL);
	if( iResult!=FZ_Success ) {
		printf("ERROR: FZ_IOCtl CMD_DE_SET_FPS failed (code 0x%02x)\n", iResult);
		goto error;
	}

	// start sensor
	iResult = FZ_IOCtl(hDevice, CMD_DE_SENSOR_START, NULL, 0, NULL, NULL, NULL);
	if( iResult!=FZ_Success ) {
		printf("ERROR: FZ_IOCtl CMD_DE_SENSOR_START failed (code 0x%02x)\n", iResult);
		goto error;
	}

	// get 200 images
	for(int i=0; i<200; i++) {
		size_t iBufSize = sizeof(aImage);
		iResult = FZ_GetFrame(hDevice, &stFrameHdr, aImage, &iBufSize);
		if( iResult!=FZ_Success ) {
			printf("ERROR: FZ_GetFrame failed (code 0x%02x)\n", iResult);
			goto error;
		}
		printf("Got frame %d\n", i);
	}

	// save the last image on disc
	bSaveOK = SaveImageAs3DD(aImage, &stFrameHdr);
	if(!bSaveOK) {
		printf("ERROR: SaveImageAs3DD failed\n");
		goto error;
	}

	// stop sensor
	FZ_IOCtl(hDevice, CMD_DE_SENSOR_STOP, NULL, 0, NULL, NULL, NULL);
	if( iResult!=FZ_Success ) {
		printf("ERROR: FZ_IOCtl CMD_DE_SENSOR_STOP failed (code 0x%02x)\n", iResult);
		goto error;
	}

	// disconnect
	FZ_Close(hDevice);

	// proper exit of FZ_API
	FZ_Exit();

	printf("ALL DONE OK\n");
	return 0;
error:
	// proper exit of FZ_API
	FZ_Exit();

	printf("Some error occured\n");
	return -1;
}
